/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "commands.h"

int cmd_copy(Client *client)
{
	Mailbox *destbox;
	const char *messageset, *mailbox;

	/* <message set> <mailbox> */
	if (!client_read_string_args(client, 2, &messageset, &mailbox))
		return FALSE;

	if (!client_verify_mailbox_name(client, mailbox, TRUE))
		return TRUE;

	/* open the destination mailbox */
	if (!client_verify_mailbox_name(client, mailbox, TRUE))
		return TRUE;

	destbox = client->storage->open_mailbox(client->storage,
						mailbox, FALSE);
	if (destbox == NULL) {
		client_send_storage_error(client);
		return TRUE;
	}

	/* copy the mail */
	if (client->mailbox->copy(client->mailbox, destbox,
				  messageset, client->cmd_uid)) {
                client_sync_mailbox(client);
		client_send_tagline(client, "OK Copy completed.");
	} else
		client_send_storage_error(client);

	destbox->close(destbox);
	return TRUE;
}
