/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "maildir-index.h"
#include "mail-index-util.h"

#include <unistd.h>
#include <fcntl.h>

int maildir_open_mail(MailIndex *index, MailIndexRecord *rec,
		      off_t *offset, size_t *size)
{
	off_t pos;
	const char *fname, *path;
	int fd;

	fname = index->lookup_field(index, rec, FIELD_TYPE_LOCATION);
	if (fname == NULL) {
                INDEX_MARK_CORRUPTED(index);
		index_set_error(index, "Corrupted index file %s: "
				"Missing location field for record %u",
				index->filepath, rec->uid);
		return -1;
	}

	path = t_strconcat(index->dir, "/cur/", fname, NULL);

	fd = open(path, O_RDONLY);
	if (fd == -1) {
		index_set_error(index, "Error opening mail file %s: %s",
				path, strerror(errno));
		return -1;
	}

	pos = lseek(fd, 0, SEEK_END);
	if (pos == (off_t)-1 || lseek(fd, 0, SEEK_SET) == (off_t)-1) {
		index_set_error(index, "lseek() failed with mail file %s: %s",
				path, strerror(errno));
		(void)close(fd);
		return -1;
	}

	*offset = 0;
	*size = (size_t) pos;
	return fd;
}
