#ifndef __INDEX_SEARCH_H
#define __INDEX_SEARCH_H

#include "imap-parser.h"

typedef enum {
	SEARCH_OR,
	SEARCH_SUB,

	/* message sets */
	SEARCH_ALL,
	SEARCH_SET,
	SEARCH_UID,

	/* flags */
	SEARCH_ANSWERED,
	SEARCH_DELETED,
	SEARCH_DRAFT,
	SEARCH_FLAGGED,
	SEARCH_SEEN,
	SEARCH_RECENT,
	SEARCH_KEYWORD,

	/* dates */
	SEARCH_BEFORE,
	SEARCH_ON,
	SEARCH_SINCE,
	SEARCH_SENTBEFORE,
	SEARCH_SENTON,
	SEARCH_SENTSINCE,

	/* sizes */
	SEARCH_SMALLER,
	SEARCH_LARGER,

	/* headers */
	SEARCH_FROM,
	SEARCH_TO,
	SEARCH_CC,
	SEARCH_BCC,
	SEARCH_SUBJECT,
	SEARCH_HEADER,

	/* body */
	SEARCH_BODY,
	SEARCH_TEXT
} SearchArgType;

typedef struct _SearchArg SearchArg;

struct _SearchArg {
	SearchArg *next;

	SearchArgType type;
	union {
		SearchArg *subargs;
		const char *str;
	} value;

	const char *hdr_value; /* for SEARCH_HEADER */
	unsigned int not:1;

	int result;
};

typedef void (*SearchForeachFunc)(SearchArg *arg, void *user_data);

/* Builds search arguments based on IMAP arguments. */
SearchArg *search_args_build(Pool pool, ImapArg *args, int args_count,
			     const char **error);

/* Reset the results in search arguments */
void search_args_reset(SearchArg *args);

/* goes through arguments in list that don't have a result yet.
   Returns 1 = search matched, -1 = search unmatched, 0 = don't know yet */
int search_args_foreach(SearchArg *args, SearchForeachFunc func,
			void *user_data);

/* Fills have_headers, have_body and have_text based on if such search
   argument exists that needs to be checked. */
void search_args_analyze(SearchArg *args, int *have_headers,
			 int *have_body, int *have_text);

#endif
